class ChessMoveChecker
{
  int[][] bord;
  int aantalrijen;
  int aantalkolommen;
  ChessCanvas thecanvas;

// booleans that tell which player may castle;
  boolean whiteshortcastling;
  boolean blackshortcastling;
  boolean whitelongcastling;
  boolean blacklongcastling;

// is it legal in this variant to make a double pawn step?
  boolean whitedoublepawnstep;
  boolean blackdoublepawnstep;

// position that white king starts: ( kingrw, 0)
  int kingrw;
// position that black king starts: ( kingrb, aantalrijen - 1)
  int kingrb;

// Which player moved last.
// 0 : white
// 1 : black
// I use an integer for doing a little easier multiplayergames

   int lastplayermoved;

// Now, we are going to turn this into an applet that plays back.
// ...
// Which player is the computer player?
   int computercolor;
// 0 : white
// 1 : black

   ComputerPlayer computerplayer;

// What is the last move: from (lastmovestartcol,lastmovestartrow) to
// (lastmoveendcol,lastmoveendrow)

   Move lastmove;
   int lastmovestartcol;
   int lastmovestartrow;
   int lastmoveendcol;
   int lastmoveendrow;

// Constructor method. Parent gives the ChessCanvas that calls this one.
// Sets the booleans whether one can castle.
// Set pointer to board of parent.
ChessMoveChecker (ChessCanvas hetbord)
  {
    bord = hetbord.bord;
    aantalrijen = hetbord.aantalrijen;
    aantalkolommen = hetbord.aantalkolommen;
    initialize();
    thecanvas = hetbord;
  }

public void initialize()
{
    whiteshortcastling = true;
    blackshortcastling = true;
    whitelongcastling = true;
    blacklongcastling = true;
    whitedoublepawnstep = true;
    blackdoublepawnstep = true;


// As white starts, that would correspond to black playing last
    lastplayermoved = 1;
    lastmovestartcol = 0; // some default. Doesn't harm I think.
    lastmovestartrow = 0; // some default. Doesn't harm I think.
    lastmoveendcol = 0;   // some default. Doesn't harm I think.
    lastmoveendrow = 0;   // some default. Doesn't harm I think.
    lastmove = new Move();

// 
    computercolor = 1;
    computerplayer = new ComputerPlayer(this);
}

// sets variables kingrw and kingrb for castling info
public void SetKingPosition()
{
// Find the place where the king starts for castling
// Standard chess setup:
    kingrw = 4;
    kingrb = 4;

    for (int i=0; i< aantalkolommen; i++)
    if ( bord[i][0]==1 ) { kingrw = i ; }
    for (int i=0; i< aantalkolommen; i++)
    if ( bord[i][aantalrijen-1]==2 ) { kingrb = i ; }
}


// 2e constructor methode. Maakt een kopie van het bord.
  ChessMoveChecker (ChessMoveChecker tocopy)
  {
    whiteshortcastling = tocopy.whiteshortcastling;
    blackshortcastling = tocopy.blackshortcastling;
    whitelongcastling = tocopy.whitelongcastling;
    blacklongcastling = tocopy.blacklongcastling;

// Standard chess setup:
    kingrw = tocopy.kingrw;
    kingrb = tocopy.kingrw;

// As white starts, that would correspond to black playing last
    lastplayermoved = tocopy.lastplayermoved;
    lastmovestartcol = tocopy.lastmovestartcol;
    lastmovestartrow = tocopy.lastmovestartrow;
    lastmoveendcol = tocopy.lastmoveendcol;
    lastmoveendrow = tocopy.lastmoveendrow;
    lastmove = new Move();

// zet als bord een pointer naar het bord van de parent.
// nog aanpassen
    aantalrijen = tocopy.aantalrijen;
    aantalkolommen = tocopy.aantalkolommen;
    bord = new int[aantalkolommen][aantalrijen];
    for (int i = 0; i< aantalkolommen; i++)
    {
      for (int j=0; j< aantalrijen; j++)
      {
	bord[i][j]=tocopy.bord[i][j];
      }
    }
  }

// test whether square (kolom, row) is a square of the board
boolean LegalSquare (int kolom, int row)
  {
// check if inside board dimensions
   if (kolom < 0) return false;
   if (kolom > aantalkolommen) return false;
   if (row < 0) return false;
   if (row > aantalrijen) return false;
// check if not a white, i.e., non-existing square
   if ( bord[kolom][row] == 1000) return false;
   return true;
  }

// test of move van veld (k1, r1) naar (k2, r2) legaal is

public boolean LegalMove (int k1, int r1, int k2, int r2)
  {
     boolean semilegal = SemiLegalMove(k1,r1,k2,r2);
     if (semilegal == false) { return false;}
     if (InCheckAfterMove(k1,r1,k2,r2)) {return false;}
     return true;
  }

public boolean LegalMove (Move themove)
  {
    return LegalMove(themove.startcolumn,
	  themove.startrow,themove.endcolumn,themove.endrow);
  }

// A move is SemiLegal if it is legal without looking for check
public boolean SemiLegalMove (int k1, int r1, int k2, int r2)
  {
    if (LegalSquare(k2,r2) == false) { return false;}
    if (k1 == k2 && r1 == r2) {return false;}
    int piece = bord[k1][r1];
    if (TakingPieceOfSameColor(k1,r1,k2,r2)) { return false;}
    if (piece == 0 ) { return false;}
    if (piece == 1 ) { return LegalWhiteKingMove (k1,r1,k2,r2);}
    if (piece == 2 ) { return LegalBlackKingMove (k1,r1,k2,r2);}
    if (piece == 3 ) { return LegalQueenMove (k1,r1,k2,r2);}
    if (piece == 4 ) { return LegalQueenMove (k1,r1,k2,r2);}
    if (piece == 5 ) { return LegalRookMove (k1,r1,k2,r2);}
    if (piece == 6 ) { return LegalRookMove (k1,r1,k2,r2);}
    if (piece == 7 ) { return LegalKnightMove (k1,r1,k2,r2);}
    if (piece == 8 ) { return LegalKnightMove (k1,r1,k2,r2);}
    if (piece == 9 ) { return LegalBishopMove (k1,r1,k2,r2);}
    if (piece == 10 ) { return LegalBishopMove (k1,r1,k2,r2);}
    if (piece == 11 ) { return LegalWhitePawnMove (k1,r1,k2,r2);}
    if (piece == 12 ) { return LegalBlackPawnMove (k1,r1,k2,r2);}
    if (piece == 13 ) { return LegalChancellorMove (k1,r1,k2,r2);}
    if (piece == 14 ) { return LegalChancellorMove (k1,r1,k2,r2);}
    if (piece == 15 ) { return LegalMarshalMove (k1,r1,k2,r2);}
    if (piece == 16 ) { return LegalMarshalMove (k1,r1,k2,r2);}
    if (piece == 31 ) { return LegalUnconvertedBishopMove (k1,r1,k2,r2);}
    if (piece == 32 ) { return LegalUnconvertedBishopMove (k1,r1,k2,r2);}
    if (piece == 33 ) { return LegalWazirMove (k1,r1,k2,r2);}
    if (piece == 34 ) { return LegalWazirMove (k1,r1,k2,r2);}
    if (piece == 1000) { return false;}
    return true;
  }

public boolean SemiLegalMove (Move themove)
  {
    return SemiLegalMove(themove.startcolumn,
	  themove.startrow,themove.endcolumn,themove.endrow);
  }

boolean LegalWhiteKingMove (int k1,int r1, int k2, int r2)
{
  // check on castling.
  if (k1==kingrw && k2==kingrw+2 && r1==0 && r2==0)
  {
    return LegalWhiteShortCastling();
  }
  if (k1==kingrw && k2==kingrw-2 && r1==0 && r2==0)
  {
    return LegalWhiteLongCastling();
  }
  return LegalKingMove(k1,r1,k2,r2);
}

public boolean LegalWhiteKingMove (Move themove)
  {
    return LegalWhiteKingMove(themove.startcolumn,
	  themove.startrow,themove.endcolumn,themove.endrow);
  }

boolean LegalBlackKingMove (int k1,int r1,int k2,int r2)
{
  // check on castling.
  if (k1==kingrw && k2==kingrw+2 && r1==aantalrijen-1 && r2==aantalrijen-1)
  {
    return LegalBlackShortCastling();
  }
  if (k1==kingrw && k2==kingrw-2 && r1==aantalrijen-1 && r2==aantalrijen-1)
  {
    return LegalBlackLongCastling();
  }
  return LegalKingMove(k1,r1,k2,r2);
}

boolean LegalKingMove (int k1,int r1,int k2,int r2)
{
   // this precisely tells if one is trying to make a kingsmove
   return ( (k1-k2)*(k1-k2) <=1 && (r1-r2)*(r1-r2) <= 1);
}

boolean LegalRookMove (int k1, int r1, int k2, int r2)
{
   if ( k1 == k2 )
   {
      boolean hulpbool = true;
      for (int i= Math.min(r1,r2)+1; i < Math.max(r1,r2); i++)
      { if ( EmptySquare(k1,i) == false) return false;}
      return true;
   }
   else if (r1 == r2)
   {
      boolean hulpbool = true;
      for (int i= Math.min(k1,k2)+1; i < Math.max(k1,k2); i++)
      { if ( EmptySquare(i,r1) == false) return false;}
      return true;
   }
   else return false;
}

boolean LegalKnightMove (int k1, int r1, int k2, int r2)
{
   int kdif = Math.abs(k1-k2);
   int rdif = Math.abs(r1-r2);
   if ((kdif == 1 && rdif == 2) || (kdif == 2 && rdif == 1))
   {
     return true;
   }
   else
   {
     return false;
   }
}

boolean LegalBishopMove (int k1, int r1, int k2, int r2)
{
   int kdif = k1-k2;
   int rdif = r1-r2;
   if (kdif == rdif || kdif == -rdif)
   {
     int kstep;
     int rstep;
     if ( k1 < k2) { kstep = 1;}
       else {kstep = -1;}
     if ( r1 < r2) { rstep = 1;}
       else {rstep = -1;}

     int i=k1+kstep; int j = r1+rstep;
     while ( i != k2 )
     {
       if (EmptySquare(i,j) == false) { return false;}
       i += kstep;
       j += rstep;
     }
   return true;
   }
   else { return false;}
}

boolean LegalUnconvertedBishopMove (int k1, int r1, int k2, int r2)
{
   return (LegalBishopMove(k1,r1,k2,r2) || LegalWazirMove(k1,r1,k2,r2));
}

boolean LegalQueenMove (int k1, int r1, int k2, int r2)
{
   return (LegalRookMove(k1,r1,k2,r2) || LegalBishopMove(k1,r1,k2,r2));
}

boolean LegalChancellorMove (int k1, int r1, int k2, int r2)
{
   return (LegalRookMove(k1,r1,k2,r2) || LegalKnightMove(k1,r1,k2,r2));
}

// Marshal moves like Bishop or Knight
boolean LegalMarshalMove (int k1,int r1, int k2, int r2)
{
   return (LegalBishopMove(k1,r1,k2,r2) || LegalKnightMove(k1,r1,k2,r2));
}

// Wazir: moves one square vertically or horizontally
boolean LegalWazirMove (int k1, int r1, int k2, int r2)
{
int kdif = Math.abs(k1-k2);
int rdif = Math.abs(r1-r2);
return ( kdif+rdif == 1 );
}

boolean LegalWhitePawnMove (int k1, int r1, int k2, int r2)
{
// no en passant capture yet
   boolean taking;
   if (bord[k2][r2] == 0 )
     {
	taking = false;
     }
   else
     {
       taking = true;
     }
   if (taking == true && r2 == r1 + 1 && ( (k1 == k2 +1) || (k1 == k2 -1)))
     {
        return true;
     }
   else if (taking == false && (r2 == r1 + 1) && k1 == k2)
     {
	return true;
     }
   else if (taking == false && r1 == 1 && r2 == 3 && k1 == k2 &&
	    bord[k1][2] == 0 )
     {
	return whitedoublepawnstep;
     }
   else if (LegalWhitePawnEnPassantCapture (k1, r1, k2, r2))
     {
        return true;
     }
   return false;
}

boolean LegalBlackPawnMove (int k1, int r1, int k2, int r2)
{
   boolean taking;
   if (bord[k2][r2] == 0 )
     {
	taking = false;
     }
   else
     {
       taking = true;
     }
   if (taking == true && r2 == r1 - 1 && ( (k1 == k2 +1) || (k1 == k2 -1)))
     {
        return true;
     }
   else if (taking == false && r2 == r1 - 1 && k1 == k2)
     {
	return true;
     }
   else if (taking == false && r1 == aantalrijen-2 && r2 == r1-2 && k1 == k2 &&
	    bord[k1][r1-1] == 0 )
     {
	return blackdoublepawnstep;
     }
   else if (LegalBlackPawnEnPassantCapture (k1, r1, k2, r2))
     {
        return true;
     }
   return false;
}

public boolean LegalWhitePawnEnPassantCapture (int k1, int r1, int k2, int r2)
{
  if (bord[k1][r1] != 11) { return false;}  // white pawn on starting square
  if (bord[k2][r2] != 0 ) { return false;}  // go to empty square
  // check if opponent made a pawn double step on destination column
  if (lastmovestartcol != k2) {return false;}
  if (lastmoveendcol != k2) {return false;}
  if (lastmovestartrow != lastmoveendrow +2) { return false;}
  if (bord[lastmoveendcol][lastmoveendrow] != 12) {return false;}
  if (r1 != aantalrijen-4) { return false;}
  if (r2 != aantalrijen-3) { return false;}
  if (Math.abs(k1-k2) != 1) {return false;}
  return true;
}

public boolean LegalBlackPawnEnPassantCapture (int k1, int r1, int k2, int r2)
{
  if (bord[k1][r1] != 12) { return false;}  // white pawn on starting square
  if (bord[k2][r2] != 0 ) { return false;}  // go to empty square
  // check if opponent made a pawn double step on destination column
  if (lastmovestartcol != k2) {return false;}
  if (lastmoveendcol != k2) {return false;}
  if (lastmovestartrow != lastmoveendrow -2) { return false;}
  if (bord[lastmoveendcol][lastmoveendrow] != 11) {return false;}
  if (r1 != 3) { return false;}
  if (r2 != 2) { return false;}
  if (Math.abs(k1-k2) != 1) {return false;}
  return true;
}

// tests if square (kolom,rij) is empty
boolean EmptySquare (int kolom, int rij)
{
   if (bord[kolom][rij] == 0)
   { 
     return true;
   }
   else
   {
     return false;
   }
}

// true if one tries to take a piece of the same color
boolean TakingPieceOfSameColor (int k1, int r1, int k2, int r2)
{
   if (EmptySquare (k2,r2)) return false;
   int p1 = bord[k1][r1] % 2;
   int p2 = bord[k2][r2] % 2;
   // p1, p2 give the player that owns the pieces on start and destination
   // squares. Rewrite for multiplayer games.
   if (p1==p2) return true;
   return false;
}

//------------------------------------------------------------------
// Methods about castling
void ChangeCastlingStatus (int k1, int r1, int k2, int r2)
{
   // if a square moves from a corner, one type of castling is never
   // possible anymore.
   if (k1==0 && r1 == 0) { whitelongcastling = false;}
   if (k1==aantalkolommen-1 && r1 == 0) { whiteshortcastling = false;}
   if (k1==0 && r1 == aantalrijen-1) {blacklongcastling = false;}
   if (k1==aantalkolommen-1 && r1 ==aantalrijen-1)
           {blackshortcastling = false;}
   if (bord[k1][r1] == 1) // white king moves
       { whitelongcastling = false;
	 whiteshortcastling = false;
       }
   if (bord[k1][r1] == 2) // white king moves
       { blacklongcastling = false;
	 blackshortcastling = false;
       }

}

// Is it legal for white to castle short?
boolean LegalWhiteShortCastling()
{
  if (InCheck(0)) return false;
  for (int i= kingrw + 1; i < aantalkolommen -1; i++)
  { 
    if (EmptySquare(i,0) == false) { return false;}
    if (Attacked(1,i,0)) { return false;}
  }
  return whiteshortcastling;
}

// is it legal for white to castle long?
boolean LegalWhiteLongCastling()
{
  if (InCheck(0)) return false;
  for (int i= kingrw - 1; i >0; i--)
  { 
    if (EmptySquare(i,0) == false) { return false;}
    if (Attacked(1,i,0)) { return false;}
  }
  return whitelongcastling;
}

  // is it legal for black to castle long?
boolean LegalBlackLongCastling()
{
  if (InCheck(1)) return false;
  for (int i= kingrw - 1; i >0; i--)
  { 
    if (EmptySquare(i,aantalrijen-1) == false) { return false;}
    if (Attacked(0,i,aantalrijen-1)) { return false;}
  }
  return blacklongcastling;
}

  // not yet finished: check for check
boolean LegalBlackShortCastling()
{
  if (InCheck(1)) return false;
  // is it legal for black to castle short?
  for (int i= kingrb + 1; i < aantalkolommen -1; i++)
  { 
    if (EmptySquare(i,aantalrijen-1) == false) { return false;}
    if (Attacked(0,i,aantalrijen-1)) { return false;}
  }
  return blackshortcastling;
}

//------------------------------------
// Determine if the move is a pawn promotion.
boolean DeterminePawnPromotion (int k1, int r1, int k2, int r2)
{
  if (bord[k1][r1]==11 && r2 == aantalrijen-1)
    {
      return true;
    }
  if (bord[k1][r1]==12 && r2 == 0)
    {
      return true;
    }
   return false;
}

boolean DeterminePawnPromotion (Move themove)
  {
     return DeterminePawnPromotion(themove.startcolumn,
               themove.startrow,themove.endcolumn,themove.endrow);
  }

public void SetLastPlayerMoved(int i)
{ 
  lastplayermoved = i;
}

public int LastPlayerMoved()
{ 
  return lastplayermoved;
}

// gets number of piece as input.
// returns 0 if white owns the piece
// returns 1 if black owns the piece
// not specified if this is not a piece.
public int PieceOwner(int i)
{
   return 1-(i%2);
}

// returns the owner of the piece on square (k,r)
public int PieceOwner(int k, int r)
{
   return PieceOwner(bord[k][r]);
}

// sets the last move

public void SetLastMove(int k1, int r1, int k2, int r2)
{
 lastmovestartcol = k1;
 lastmovestartrow = r1;
 lastmoveendcol = k2;
 lastmoveendrow = r2;
}

// checks if the right player wants to make a move
public boolean RightPlayer(int k1, int r1, int k2, int r2)
{
   return (PieceOwner(k1,r1) != LastPlayerMoved());
}

public boolean RightPlayer(int k1, int r1)
{
   return (PieceOwner(k1,r1) != LastPlayerMoved());
}


// Checks if player is in check
boolean InCheck (int player)
{
// first find the king piece of the player.
  int kingpiece = 1 + player;
  for (int k=0; k< aantalkolommen; k++)
  {
    for (int r=0; r < aantalrijen; r++)
    {
       if (bord[k][r]==kingpiece)
       {
	 // crude: check if any piece could move to the kings position
	 for (int k2=0; k2<aantalkolommen; k2++)
	 {
	   for (int r2=0; r2<aantalrijen; r2++)
	   {
	     if ((bord[k2][r2] != 0) || (bord[k2][r2] !=1000))
	     {
	       if (SemiLegalMove(k2,r2,k,r)) 
	       {
		 return true;
               }
	     }
	   }
	 }
       }
    }
  }
  return false;
}

// has player a semilegal move that attacks (k,r)
boolean Attacked (int player, int k, int r)
{
// unwritten
for (int k2=0; k2 < aantalkolommen; k2++)
{
   for (int r2=0; r2 < aantalrijen; r2++)
   {
     if ((! EmptySquare(k2,r2)) || (bord[k2][r2]!=1000) )
     {
	if (PieceOwner(k2,r2)== player)
	{
	   if (SemiLegalMove(k2,r2,k,r))
	   {
	      return true;
	   }
	}
     }
   }
}
return false;
}

boolean InCheckAfterMove (int k1, int r1, int k2, int r2)
{
  int player = PieceOwner(k1,r1);
  ChessMoveChecker aftermove = new ChessMoveChecker(this);
  aftermove.DoMove(k1,r1,k2,r2);
  return aftermove.InCheck(player);
}

void DoMove (int k1, int r1, int k2, int r2)
{
// First determine special types of moves
   // determine if this is a promotion move
   boolean promotion = DeterminePawnPromotion(k1,r1,k2,r2);
   // determine if this is a castling move
   boolean castled;
   if (bord[k1][r1] > 2) { castled = false;}
   else if (Math.abs(k1-k2) < 2) {castled = false;}
   else {castled = true;}
   boolean whiteenpassant = LegalWhitePawnEnPassantCapture (k1, r1, k2, r2);
   boolean blackenpassant = LegalBlackPawnEnPassantCapture (k1, r1, k2, r2);

// update status
   SetLastMove(k1,r1,k2,r2);
   SetLastPlayerMoved(PieceOwner(k1,r1));
   ChangeCastlingStatus(k1,r1,k2,r2);

// do the move
   bord[k2][r2] = bord[k1][r1];
   bord[k1][r1]=0;

   // additional work if promotion
   if (promotion)
   {
     int speler = 1-bord[k2][r2]%2;
     bord[k2][r2] = DeterminePromotionPiece(speler);
   }
   // additional work if castling
   if (castled)
   {
     HandleCastling(k1,r1,k2,r2);
   }
   if (whiteenpassant)
   {
     bord[k2][r2-1]=0; // remove captured pawn
   }
   if (blackenpassant)
   {
     bord[k2][r2+1]=0; // remove captured pawn
   }

   HandleBCR(k1,r1,k2,r2);

}

// Handle the bishop conversion rule.
void HandleBCR(int k1, int r1, int k2, int r2)
{
   if (bord[k2][r2] == 31) // unconverted bishop
   {
    // change the bishop to a normal bishop
    bord[k2][r2] = 9;
    // Find a possible other unconverted bishop and make it a 33
    for (int i=0; i<aantalkolommen; i++)
    { for (int j=0; j<aantalrijen; j++)
      { if (bord[i][j]==31) { bord[i][j]=33;}
      }
    }
   }
   if (bord[k2][r2] == 32) // unconverted bishop
   {
    // change the bishop to a normal bishop
    bord[k2][r2] = 10;
    // Find a possible other unconverted bishop and make it a 34
    for (int i=0; i<aantalkolommen; i++)
    { for (int j=0; j<aantalrijen; j++)
      { if (bord[i][j]==32) { bord[i][j]=34;}
      }
    }
   }
// change wazir and then bishop to bishop
   if (bord[k2][r2] == 33) bord[k2][r2]=9;
   if (bord[k2][r2] == 34) bord[k2][r2]=10;
}

public void MakeComputerMove()
{
     Move computermove = ComputerPlayer.findmove(this,computercolor);
     if (!computermove.isvoid())
     {DoMove(computermove);}
}


void DoMove (Move themove)
  {
      DoMove(themove.startcolumn,
		themove.startrow,themove.endcolumn,themove.endrow);
  }


// modify later for including Fisher Random Chess.
void HandleCastling(int k1,int r1,int k2,int r2)
{
   if (k1 > k2)                           // castling to left of board
   { 
     bord[k2+1][r2] = bord[0][r2];
     bord[0][r2] = 0;
   }
   else                                   // castling to right of board
   { 
     bord[k2-1][r2] = bord[aantalkolommen -1][r2];
     bord[aantalkolommen-1][r2]=0;
   }
}

void HandleCastling (Move themove)
  {
      HandleCastling(themove.startcolumn,
		themove.startrow,themove.endcolumn,themove.endrow);
  }

// Determineert naar welk stuk gepromoveerd wil worden
// Erg rudimentair: alleen dames
// speler: 0 wit; 1 zwart
int DeterminePromotionPiece (int speler)
{
  int stuk = 3+speler;
  return stuk;
}

//====================================================================

// Methods for variants

public void DisableCastling()
{
  whiteshortcastling = false;
  blackshortcastling = false;
  whitelongcastling = false;
  blacklongcastling = false;
}

public void DisableDoublePawnStep()
{
  whitedoublepawnstep = false;
  blackdoublepawnstep = false;
}

public void DisableDoublePawnStep(int player)
{
  if (player == 0) whitedoublepawnstep = false;
  if (player == 1) blackdoublepawnstep = false;
}

boolean Mated(int player)
{
  if (lastplayermoved != player)
  {
    if (InCheck(player))
    {
      return (!HasALegalMove(player));
    }
  }
  return false;
}

boolean Stalemated(int player)
{
  if (lastplayermoved != player)
  {
    if (!InCheck(player))
    {
      return (!HasALegalMove(player));
    }
  }
  return false;
}


boolean HasALegalMove(int player)
{
  for (int k=0; k<aantalkolommen; k++)
   {
    for (int r=0; r<aantalrijen;r++)
    {
     if ((!EmptySquare(k,r)) || (bord[k][r] != 1000))
     {
      if (PieceOwner(k,r)==player)
      {
       for (int i2=0; i2<aantalkolommen; i2++)
       {
        for (int j2=0; j2<aantalrijen; j2++)
        {
	     if (LegalMove(k,r,i2,j2))
	     {
	      return true;
         }
        }
       }
      }
     }
    }
   }
   return false;
}

// END OF CLASS
}
